/***********************************************************************

  FinalMix: DeviceMM.cpp
  by Martin Decky
  
  Redistribution of this file is permitted under the GNU Public License.

  0.9.2		Initial
  0.9.4		Complete rewrite, native threads instead of wxThread

***********************************************************************/

#include "generic.h"
#include "DeviceMM.h"

#if defined(__WXMSW__)

#define FM_MSG_SIGN WM_USER + 1

DWORD __stdcall play_thread(LPVOID data) {
	MMDevice * parent = (MMDevice * ) data;
	MSG msg;
	samples_t second = 2 * 2 * 44100; // FIXME
	samples_t fgs;
	
	while (GetMessage(&msg, NULL, 0, 0)) {
		fgs = 1;
		switch (msg.message) {
			case FM_MSG_SIGN:
				fgs = 2;
			case MM_WOM_DONE:
				if (parent->opened) {
					DWORD sleep;

					while (fgs > 0) {

						WaitForSingleObject(parent->buf_mutex, INFINITE);

						if (parent->buf_size > 0) {
							if (waveOutWrite(parent->handle, parent->fragments + parent->buf_start, sizeof(WAVEHDR)) == MMSYSERR_NOERROR) {
								parent->buf_start++;
								parent->buf_size--;
								parent->position += parent->latency;
								if (parent->buf_start == parent->buf_count)
									parent->buf_start = 0;
								if (parent->buf_size < (parent->buf_count / 2))
									PostThreadMessage(parent->push_id, FM_MSG_SIGN, 0, 0);
							}
							sleep = 0;
						} else {
							PostThreadMessage(parent->push_id, FM_MSG_SIGN, 0, 0);
							normsample_t flt = (normsample_t) parent->latency / (normsample_t) (__int64) second / 2;	/* MSVC can't convert unsigned __int64 to double */
							sleep = (DWORD) (flt * 1000);
						}
						
						ReleaseMutex(parent->buf_mutex);
						Sleep(sleep);
						fgs--;
					}
				}
				break;
			default:
				break;
		}
	}

	return 0;
}

DWORD __stdcall push_thread(LPVOID data) {
	MMDevice * parent = (MMDevice * ) data;
	MSG msg;

	while (GetMessage(&msg, NULL, 0, 0)) {
		switch (msg.message) {
			case FM_MSG_SIGN:
				size_t tr;
				size_t len;

				WaitForSingleObject(parent->buf_mutex, INFINITE);
		
				tr = parent->buf_count - parent->buf_size;
		
				if (tr > 0) {
					len = parent->cb->OnBuffer(parent->tmpbuf, tr * parent->latency / SampleLen(parent->pfmt) / parent->pfmt.channels, (parent->position + parent->buf_size) / SampleLen(parent->pfmt) / parent->pfmt.channels, parent->pfmt) * SampleLen(parent->pfmt) * parent->pfmt.channels / parent->latency;
					if (len > 0) {
		
						size_t tr;
						size_t rd;
						size_t off = 0;
						size_t tlen = len;
			
						if ((parent->buf_size < parent->buf_count) && (parent->buf_start + parent->buf_size < parent->buf_count)) {
							tr = parent->buf_count - parent->buf_start - parent->buf_size;
							rd = min(tr, tlen);
							memcpy((char * ) parent->buf + (parent->buf_start + parent->buf_size) * parent->latency, (char * ) parent->tmpbuf + off * parent->latency, rd * parent->latency);
							parent->buf_size += rd;
							tlen -= rd;
							off += rd;
						}
						if ((parent->buf_size < parent->buf_count) && (parent->buf_start + parent->buf_size >= parent->buf_count) && (parent->buf_start > 0)) {
							tr = parent->buf_count - parent->buf_size;
							rd = min(tr, tlen);
							memcpy((char * ) parent->buf + (parent->buf_start + parent->buf_size - parent->buf_count) * parent->latency, (char * ) parent->tmpbuf + off * parent->latency, rd * parent->latency);
							parent->buf_size += rd;
						}

						bool ret = false;
						while (!ret) {
							ret = PostThreadMessage(parent->play_id, FM_MSG_SIGN, 0, 0);
							if (!ret)
								Sleep(100);
						}
				
					}
				}

				ReleaseMutex(parent->buf_mutex);
				break;
			default:
				break;
		}
	}

	return 0;
}

MMDevice::MMDevice() {
	buf = NULL;
	tmpbuf = NULL;
	fragments = NULL;
	opened = false;

	buf_mutex = CreateMutex(NULL, false, NULL);
}

MMDevice::~MMDevice() {
	done();

	CloseHandle(buf_mutex);
}

wxString MMDevice::Name() const {
	return("Windows MM device");
}

bool MMDevice::Init(const format_ex_t fmt, DeviceCallback & callback) {
	done();
	
	cb = &callback;
	pfmt = fmt;
	
	buf_count = 10; // FIXME configure
	latency = 10240; // FIXME configure, must be a multiply of 1024
		
	buf_start = 0;
	buf_size = 0;
	position = 0;
	opened = false;
	playing = false;

	WAVEOUTCAPS wavecaps;
	
	if (waveOutGetDevCaps(0, &wavecaps, sizeof(wavecaps)) != MMSYSERR_NOERROR) { // FIXME config (0)
		done();
		return false;
	}

	if (wavecaps.dwSupport & WAVECAPS_SYNC == 0) {
		done();
		return false;
	}
	
	fragments = new WAVEHDR[buf_count];
	if (!fragments) {
		done();
		return false;
	}
	memset(fragments, 0, sizeof(WAVEHDR) * buf_count);

	buf = new char[buf_count * latency];
	if (!buf) {
		done();
		return false;
	}
	
	tmpbuf = new char[buf_count * latency];
	if (!tmpbuf) {
		done();
		return false;
	}

	play = CreateThread(NULL, 0, play_thread, this, 0, &play_id);
	push = CreateThread(NULL, 0, push_thread, this, 0, &push_id);

	WaitForSingleObject(buf_mutex, INFINITE);
		
	WAVEFORMATEX wfx;
	memset(&wfx, 0, sizeof(wfx));
	wfx.wFormatTag = WAVE_FORMAT_PCM;
	wfx.nChannels = 2; // FIXME config
	wfx.nSamplesPerSec = 44100; // FIXME config
	wfx.nAvgBytesPerSec = 4 * 44100;
	wfx.nBlockAlign = 4;
	wfx.wBitsPerSample = 16; // FIXME config
	wfx.cbSize = sizeof(wfx);

	if (waveOutOpen(&handle, 0, &wfx, (DWORD) play_id, 0, CALLBACK_THREAD) != MMSYSERR_NOERROR) {
		done();
		return false;
	}

	for (size_t frag = 0; frag < buf_count; frag++) {
		fragments[frag].dwBufferLength = latency;
		fragments[frag].lpData = (char * ) buf + frag * latency;
		if (waveOutPrepareHeader(handle, fragments + frag, sizeof(WAVEHDR)) != MMSYSERR_NOERROR) {
			done();		// FIXME unprepare
			return false;
		}
	}
		
	opened = true;

	ReleaseMutex(buf_mutex);

	return true;
}

bool MMDevice::Play() {
	playing = true;

	bool ret = false;
	while (!ret) {
		ret = PostThreadMessage(push_id, FM_MSG_SIGN, 0, 0);
		if (!ret)
			Sleep(100);
	}

	return true;
}

bool MMDevice::Pause() {
	// FIXME

	if (playing)
		playing = false;
	else {
		playing = true;
//		PostThreadMessage(play_id, FM_MSG_SIGN, 0, 0);
		PostThreadMessage(push_id, FM_MSG_SIGN, 0, 0);
	}
	return true;
}

bool MMDevice::Stop() {
	if (playing)
		playing = false;

	done();

	return true;
}

bool MMDevice::Done() {
	if (playing)
		playing = false;

	done();

	return true;
}

samples_t MMDevice::GetPosition() {
	if (opened) {											/* is device initialized? */
		WaitForSingleObject(buf_mutex, INFINITE);			/* enter buffer critical section */
		samples_t pos = position;
		ReleaseMutex(buf_mutex);							/* leave buffer critical section */
		return (pos / SampleLen(pfmt));						/* position in samples */
	} else
		return 0;
}

Device * MMDevice::RegisterDevice() {
	return (new MMDevice);
}

void MMDevice::done() {
	WaitForSingleObject(buf_mutex, INFINITE);
	
	// FIXME check if created
	PostThreadMessage(play_id, WM_QUIT, 0, 0);
	PostThreadMessage(push_id, WM_QUIT, 0, 0);

	if (tmpbuf) {
		delete[] (char * ) tmpbuf;
		buf = NULL;
	}
	
	if (buf) {
		delete[] (char * ) buf;
		buf = NULL;
	}

	if (fragments) {
		delete[] fragments;
		fragments = NULL;
	}
	
	opened = false;
	// FIXME close device
		
	ReleaseMutex(buf_mutex);
}

#endif
