/***********************************************************************

  FinalMix: DeviceOSS.h
  by Martin Decky
  
  Redistribution of this file is permitted under the GNU Public License.

  0.9.2		Initial
  0.9.4		Complete rewrite, native pthreads instead of wxThread

**********************************************************************/

#include "Devices.h"
#include <wx/wx.h>

#if defined(__WXGTK__) || defined(__WXMOTIF__)

#ifndef _FM_DEVICEOSS_H_
#define _FM_DEVICEOSS_H_

#include <pthread.h>
#include <semaphore.h>

class OSSDevice : public Device {
	public:
		OSSDevice();
		virtual ~OSSDevice();
	
		virtual wxString Name() const;
		
		static Device * RegisterDevice();
		
		virtual bool Init(const format_ex_t fmt, DeviceCallback & callback);
		virtual bool Play();
		virtual bool Pause();
		virtual bool Stop();
		virtual bool Done();
		
		virtual samples_t GetPosition();
		
		friend void * play_thread(void * data);
		friend void * push_thread(void * data);
		
	private:
		void done();
		
		DeviceCallback * cb;
		format_ex_t pfmt;
		
		pthread_t play;
		pthread_t push;
		
		int pdev;
		bool opened;
		bool playing;
		
		sem_t play_sem;
		sem_t push_sem;
		
		pthread_mutex_t buf_mutex;
		size_t buf_count;
		size_t buf_len;
		size_t buf_start;
		size_t buf_size;
		size_t latency;
		void * buf;
		void * tmpbuf;
		samples_t position;
};

#endif

#endif
