/***********************************************************************

  FinalMix: Dialogs.cpp
  by Martin Decky
  
  Redistribution of this file is permitted under the GNU Public License.

  0.9.1		All general dialogs will be here
  0.9.2		Track selection dialog

***********************************************************************/

#include "generic.h"
#include "Dialogs.h"

#define ID_TDLG_LISTBOX (wxID_HIGHEST + 1)

class TrackDlg : public wxDialog {
	public:
		TrackDlg() {};
		TrackDlg(wxWindow * parent, const wxString & title, const wxString & mesg, const Loader * ld);
		~TrackDlg();
		
		void SetSelections(const wxArrayInt & selections);
		wxArrayInt GetSelections();
		void OnListbox(wxCommandEvent & event);
	protected:
		wxArrayInt selections;
		wxString * choices;
		wxListBox * listbox;
		wxStaticText * info;
		const Loader * loader;
	private:
		DECLARE_DYNAMIC_CLASS(TrackDlg)
		DECLARE_EVENT_TABLE()
};

bool TrackChoices(wxArrayInt & selections, const wxString & message, const wxString & caption, const Loader * ld, wxWindow * parent) {
	if (ld == NULL)
		return(false);
	
	TrackDlg dialog(parent, caption, message, ld);
	
	if (!selections.IsEmpty())
		dialog.SetSelections(selections);
	
	if (dialog.ShowModal() == wxID_OK)
		selections = dialog.GetSelections();
	else
		selections.Empty();
	
	return(true);
}

BEGIN_EVENT_TABLE(TrackDlg, wxDialog)
	EVT_LISTBOX(ID_TDLG_LISTBOX, TrackDlg::OnListbox)
END_EVENT_TABLE()

IMPLEMENT_DYNAMIC_CLASS(TrackDlg, wxDialog)

TrackDlg::TrackDlg(wxWindow * parent, const wxString & title, const wxString & mesg, const Loader * ld)
	: wxDialog(parent, -1, title, wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxDIALOG_MODAL | wxRESIZE_BORDER | wxTAB_TRAVERSAL) {
	
	ASSERT(ld != NULL);
	
	loader = ld;
	choices = new wxString[loader->NumIntTracks()];
	for (track_t i = 0; i < loader->NumIntTracks(); i++)
		choices[i] << wxString("Track ") << (i + 1);
	
	wxBoxSizer * topsizer = new wxBoxSizer(wxVERTICAL);
	topsizer->Add(CreateTextSizer(mesg), 0, wxALL, 10);
	listbox = new wxListBox(this, ID_TDLG_LISTBOX, wxDefaultPosition, wxDefaultSize, loader->NumIntTracks(), choices, wxLB_ALWAYS_SB | wxLB_EXTENDED);
	topsizer->Add(listbox, 1, wxEXPAND|wxLEFT | wxRIGHT,15);
	info = new wxStaticText(this, -1, "", wxDefaultPosition);
	topsizer->Add(info, 1, wxEXPAND|wxLEFT | wxRIGHT,15);
	topsizer->Add(CreateButtonSizer(wxOK | wxCANCEL), 0, wxCENTRE | wxALL, 10);
	SetAutoLayout(true);
	SetSizer(topsizer);
	topsizer->SetSizeHints(this);
	topsizer->Fit(this);
	Centre(wxBOTH);
	listbox->SetFocus();

	wxCommandEvent dummy;
	OnListbox(dummy);
}

TrackDlg::~TrackDlg(void) {
	delete info;
	delete listbox;
}

void TrackDlg::SetSelections(const wxArrayInt & selections) {
	size_t count = selections.GetCount();
	for (size_t n = 0; n < count; n++) {
		listbox->Select(selections[n]);
	}
}

wxArrayInt TrackDlg::GetSelections() {
	listbox->GetSelections(selections);
	return(selections);
}

void TrackDlg::OnListbox(wxCommandEvent & WXUNUSED(event)) {
	int i = listbox->GetSelection();

	if (i >= loader->NumIntTracks())
		i = 0;
	
	if ((i >= 0) && (i < loader->NumIntTracks())) {
		wxString mesg;
		const format_t * data = loader->TrackFormat(i);
		
		ASSERT(data != NULL);
		
		mesg << "Sample rate: " << data->rate << " Hz\nEncoding: ";
		switch (data->samplewidth) {
			case BIT8:
				mesg += "8";
				break;
			case BIT16:
				mesg += "16";
				break;
		}
		mesg += " bit";
	 	switch (data->bitencoding) {
			case UNSIGNED:
				mesg += " unsigned";
				break;
			case SIGNED:
				mesg += " signed";
				break;
		}
		switch (data->encoding) {
			case PCM:
				mesg += " (PCM)";
				break;
	 	}
		info->SetLabel(mesg);
	} else
		info->SetLabel("");
}
