/***********************************************************************

  FinalMix: Draw.cpp
  by Martin Decky
  
  Redistribution of this file is permitted under the GNU Public License.

  0.9.0		Initial version
  0.9.1		No change
  0.9.2		No change

***********************************************************************/

#include "generic.h"
#include "Draw.h"

bool Draw::inited = false;
wxBrush Draw::lightBrush[2];
wxBrush Draw::mediumBrush[2];
wxBrush Draw::darkBrush[2];
wxPen Draw::lightPen[2];
wxPen Draw::mediumPen[2];
wxPen Draw::darkPen[2];
wxPen Draw::peakPen;
wxPen Draw::xcursorPen;

void Draw::Bevel(wxDC & dc, bool up, wxRect & r) {
	if (up)
		Draw::Light(&dc, false);
    else
		Draw::Dark(&dc, false);
    dc.DrawLine(r.x, r.y, r.x + r.width, r.y);
    dc.DrawLine(r.x, r.y, r.x, r.y + r.height);

    if (!up)
		Draw::Light(&dc, false);
    else
		Draw::Dark(&dc, false);
    dc.DrawLine(r.x + r.width, r.y, r.x + r.width, r.y + r.height);
    dc.DrawLine(r.x, r.y + r.height, r.x + r.width + 1, r.y + r.height);
}

void Draw::Light(wxDC * dc, bool selected) {
    if (!inited)
		Init();
	
	if (selected) {
		dc->SetBrush(lightBrush[1]);
		dc->SetPen(lightPen[1]);
	} else {
		dc->SetBrush(lightBrush[0]);
    	dc->SetPen(lightPen[0]);
	}
}

void Draw::Medium(wxDC * dc, bool selected) {
    if (!inited)
		Init();
		
	if (selected) {
		dc->SetBrush(mediumBrush[1]);
		dc->SetPen(mediumPen[1]);
	} else {
		dc->SetBrush(mediumBrush[0]);
    	dc->SetPen(mediumPen[0]);
	}
}

void Draw::Dark(wxDC * dc, bool selected) {
    if (!inited)
		Init();
	
	if (selected) {	
		dc->SetBrush(darkBrush[1]);
		dc->SetPen(darkPen[1]);
	} else {
		dc->SetBrush(darkBrush[0]);
		dc->SetPen(darkPen[0]);
	}
}

void Draw::Peak(wxDC * dc) {
	if (!inited)
		Init();

	dc->SetPen(peakPen);
}

void Draw::XCursor(wxDC * dc) {
	if (!inited)
		Init();

	dc->SetPen(xcursorPen);
}

void Draw::Init() {
    if (inited)
	return;

    wxColour light = wxSystemSettings::GetSystemColour(wxSYS_COLOUR_3DHIGHLIGHT);
    wxColour med = wxSystemSettings::GetSystemColour(wxSYS_COLOUR_3DFACE);
    wxColour dark = wxSystemSettings::GetSystemColour(wxSYS_COLOUR_3DSHADOW);

    lightBrush[0].SetColour(light);
    mediumBrush[0].SetColour(med);
    darkBrush[0].SetColour(dark);
    lightPen[0].SetColour(light);
    mediumPen[0].SetColour(med);
    darkPen[0].SetColour(dark);
    lightBrush[1].SetColour(204,204,255);
    mediumBrush[1].SetColour(200,200,214);
    darkBrush[1].SetColour(148,148,170);
    lightPen[1].SetColour(204,204,255);
    mediumPen[1].SetColour(200,200,214);
    darkPen[1].SetColour(0,0,0);
	
	peakPen.SetColour(224, 76, 76);
	xcursorPen.SetColour(163, 155, 255);

    inited = true;
}
