/***********************************************************************

  FinalMix: Frame.h
  by Martin Decky
  
  Redistribution of this file is permitted under the GNU Public License.

  0.9.0		Initial version
  0.9.1		Configuration
  0.9.2		Cleanup

**********************************************************************/

#ifndef _FM_FRAME_H_
#define _FM_FRAME_H_

#include <wx/frame.h>
#include <wx/menu.h>
#include <wx/config.h>
#include <wx/scrolbar.h>
#include <wx/toolbar.h>
#include <wx/bitmap.h>
#include <wx/icon.h>
#include <wx/bmpbuttn.h>
#include <wx/timer.h>

enum {
	ID_New = 1,
	ID_Open,
	ID_Save,
	ID_SaveAs,
	ID_Revert,
	ID_Render,
	ID_Quit,
	ID_Undo,
	ID_Redo,
	ID_Cut,
	ID_Copy,
	ID_PasteMenu,
	ID_PasteOver,
	ID_PasteInto,
	ID_PasteNewTrack,
	ID_Add,
	ID_Import,
	ID_Flatten,
	ID_Settings,
	ID_Help,
	ID_Tips,
	ID_About,
	ID_Timer,
	ID_PlayBut,
	ID_PauseBut,
	ID_StopBut,
	ID_FDynBut
};

class Controls;

class Area : public wxScrolledWindow {
	public:
		Area(wxFrame * parent, Controls * controls);
	
		void OnPaint(wxPaintEvent & event);
		void OnScroll(wxScrollWinEvent & event);
		void Update();

		DECLARE_EVENT_TABLE()
		
	private:
		Controls * ctrl;
};

class Controls : public wxWindow {
	public:
		Controls(wxFrame * parent);
		void CScroll(const int x, const int y);
		void Update();
		void OnMute(wxCommandEvent & event);
		
		DECLARE_EVENT_TABLE()
		
	private:
		wxPoint origin;
		wxStaticBox * boxes;
		wxButton * mutes;
};

class Frame : public wxFrame {
	public:
		Frame(const wxString & title);
		~Frame(void);
		
		void OnQuit(wxCommandEvent & event);
		void OnAbout(wxCommandEvent & event);
		void OnTips(wxCommandEvent & event);
		void OnOpen(wxCommandEvent & event);
		void OnSaveAs(wxCommandEvent & event);
		void OnImport(wxCommandEvent & event);
		
		void OnPlayBut(wxCommandEvent & event);
		void OnPauseBut(wxCommandEvent & event);
		void OnStopBut(wxCommandEvent & event);
		
		void OnTimer(wxTimerEvent & event);
	
		void ShowTips(bool force);
	
		void Update();
		
		DECLARE_EVENT_TABLE()
		
	private:
		wxMenuBar * menuBar;
		wxMenu * menuFile;
		wxMenu * menuEdit;
		wxMenu * submenuPaste;
		wxMenu * menuTracks;
		wxMenu * menuOptions;
		wxMenu * menuHelp;
		wxToolBar * toolBar;
		wxConfigBase * configure;
		wxBitmapButton * playBut;
		wxBitmapButton * pauseBut;
		wxBitmapButton * stopBut;
	
		Area * area;
		Controls * controls;
		wxTimer * timer;
		wxBoxSizer * cbox;
};

#endif
