/***********************************************************************

  FinalMix: Peaks.h
  by Martin Decky
  
  Redistribution of this file is permitted under the GNU Public License.

  0.9.2		Initial

**********************************************************************/

#ifndef _FM_PEAKS_H_
#define _FM_PEAKS_H_

#include "generic.h"
#include "Loaders.h"
#include <wx/datetime.h>

#define PEAK_MAX 1.0

/** Peaks class.
 *
 * Peak class is used to quickly access audio track peaks to display the
 * waveform on screen. The class uses memory and disk cache for the
 * peak data and maintains persistent peak file for each track.
 */

class Peaks {
	public:
		
		/** Constructor.
		 *
		 * Initializes the peak persistnce.
		 *
		 * @param ld      Loader holding the track.
		 * @param itrack  Intex of the subtrack for the loader.
		 * @param name    Filename of the persistent peak file.
		 * @param density FIXME
		 *
		 */
	
		Peaks(Loader * ld, const track_t itrack, const wxString & name, const ratio_t density);
		~Peaks();																				/**< Destructor */
		
		bool Synchronize(wxWindow * parent);													/**< FIXME */
		bool GetPeak(const samples_t ppos, const ratio_t density, normsample_t * peak);			/**< FIXME */
	private:
		Loader * loader;																		/**< Loader */
		track_t track;																			/**< Subtrack */
		bool empty;
		wxFile * peaks;																			/**< Peaks persistent file */
		ratio_t ratio;																			/**< Display ratio */
		off_t stamppos;																			/**< FIXME */
		off_t datapos;																			/**< FIXME */
		
		bool write_header(const wxDateTime modtime, const length_t length);						/**< FIXME */
		bool check_header(const wxDateTime modtime, const length_t length);						/**< FIXME */
		bool write_string(const char * str);													/**< FIXME */
		bool write_tag(const char * str);														/**< FIXME */
		bool read_tag(char * str, size_t size);													/**< FIXME */
		
		void not_persistent();																	/**< FIXME */
};

#endif
